//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <pglabel.h>
#include <pgapplication.h>

#include "PlayerDialog.h"
#include "../playerlist.h"
#include "../stacklist.h"
#include "../citylist.h"
#include "../File.h"
#include "../defs.h"


E_PlayerDialog::E_PlayerDialog(PG_Widget* parent, PG_Rect rect)
    :PG_Window(parent, rect, _("Manage players"), PG_Window::MODAL), d_current(0)
{
    // - at the top, we but the player, player type and armyset selections
    // - below comes the player name, gold and color + a checkbox
    // - at the bottom are the buttons (add, remove, ok)

    // First the top

    // player selection, fill it with all players
    PG_Rect r(30, 50, 150, 20);
    d_players = new PG_DropDown(this, r);
    d_players->sigSelectItem.connect(slot(*this, &E_PlayerDialog::playerSelected));

    // selection of player types
    r.x += 170; r.w = 100;
    d_types = new PG_DropDown(this, r);
    d_types->AddItem(_("Human"));
    d_types->AddItem(_("Dummy AI"));
    d_types->AddItem(_("Fast AI"));
    d_types->AddItem(_("Smart AI"));
    d_types->SetEditable(false);
    d_types->sigSelectItem.connect(slot(*this, &E_PlayerDialog::typeChanged));

    // selection of armyset
    r.x += 120;
    d_armysets = new PG_DropDown(this, r);
    std::list<std::string> sets = File::scanArmysets();
    for (std::list<std::string>::iterator it = sets.begin(); it != sets.end(); it++)
    {
        if ((*it) == "default" || (*it) == "heroes")
            continue;
        d_armysets->AddItem((*it).c_str());
    }
    d_armysets->SetEditable(false);
    d_armysets->sigSelectItem.connect(slot(*this, &E_PlayerDialog::armysetChanged));
   
    
    // now the middle part

    // the player name and gold
    r.SetRect(30, 100, 60, 20);
    PG_Label* label = new PG_Label(this, r, _("Name:"));
    r.y += 30;
    label = new PG_Label(this, r, _("Gold:"));

    r.SetRect(100, 100, 150, 20);
    d_l_name = new PG_LineEdit(this, r);
    d_l_name->sigEditEnd.connect(slot(*this, &E_PlayerDialog::valueChanged));
    r.y += 30;
    d_l_gold = new PG_LineEdit(this, r, "LineEdit", 5);
    d_l_gold->SetValidKeys("0123456789");
    d_l_gold->sigEditEnd.connect(slot(*this, &E_PlayerDialog::valueChanged));

    // the player color
    label = new PG_Label(this, PG_Rect(30, 170, 60, 20), _("Color"));

    r.SetRect(100, 170, 40, 20);
    label = new PG_Label(this, r, _("R:"));
    r.x += 45;
    d_l_red = new PG_LineEdit(this, r, "LineEdit", 3);
    d_l_red->SetValidKeys("0123456789");
    d_l_red->sigEditEnd.connect(slot(*this, &E_PlayerDialog::valueChanged));
    r.x += 50;
    label = new PG_Label(this, r, _("G:"));
    r.x += 45;
    d_l_green = new PG_LineEdit(this, r, "LineEdit", 3);
    d_l_green->SetValidKeys("0123456789");
    d_l_green->sigEditEnd.connect(slot(*this, &E_PlayerDialog::valueChanged));
    r.x += 50;
    label = new PG_Label(this, r, _("B:"));
    r.x += 45;
    d_l_blue = new PG_LineEdit(this, r, "LineEdit", 3);
    d_l_blue->SetValidKeys("0123456789");
    d_l_blue->sigEditEnd.connect(slot(*this, &E_PlayerDialog::valueChanged));
    
    // plus a label for the id
    // and a checkbox that indicates whether the player is the neutral one,
    // this one cannot be deleted or set to be human or such.
    r.SetRect(280, 100, 50, 20);
    label = new PG_Label(this, r, _("ID:"));
    r.x += r.w + 10;
    d_l_id = new PG_Label(this, r, "");
    
    r.SetRect(280, 130, 150, 20);
    d_neutral = new PG_CheckButton(this, r, _("neutral player"));
    d_neutral->EnableReceiver(false);


    // Finally, the buttons at the bottom

    // the button to add a player
    r.SetRect(my_width/2-165, my_height-50, 90, 30);
    d_b_add = new PG_Button(this, r, _("Add"));
    d_b_add->sigClick.connect(slot(*this, &E_PlayerDialog::playerAdded));

    r.x += 110;
    d_b_remove = new PG_Button(this, r, _("Remove"));
    d_b_remove->sigClick.connect(slot(*this, &E_PlayerDialog::playerRemoved));

    r.x += 110;
    PG_Button* b_ok = new PG_Button(this, r, _("OK"));
    b_ok->sigClick.connect(slot(*this, &E_PlayerDialog::okClicked));


    // initialize with the first player
    refillPlayers();
    d_players->SelectFirstItem();
}

E_PlayerDialog::~E_PlayerDialog()
{
}

bool E_PlayerDialog::okClicked(PG_Button* btn)
{
    QuitModal();
    return true;
}

bool E_PlayerDialog::playerRemoved(PG_Button* btn)
{
    Playerlist* pl = Playerlist::getInstance();
    
    if (!d_current || d_current == pl->getNeutral())
        return true;

    // we need to remove all things which the player might possess. These are
    // especially cities and stacks. For simplicity, we just have them taken
    // over by the neutral player
    Player* neutral = pl->getNeutral();
    Stacklist* sl = d_current->getStacklist();
    while (!sl->empty())
    {
        neutral->addStack(*(sl->begin()));
        sl->erase(sl->begin());
    }

    Citylist* cl = Citylist::getInstance();
    for (Citylist::iterator it = cl->begin(); it != cl->end(); it++)
        if ((*it).getPlayer() == d_current)
            (*it).conquer(neutral);

    //TODO: remove all events associated with this player lateron
    
    
    // remove the player from the playerlist etc.
    for (Playerlist::iterator it = pl->begin(); it != pl->end(); it++)
        if (d_current == (*it))
        {
            pl->erase(it);
            delete d_current;
            d_current = 0;
            break;
        }

    refillPlayers();
    
    d_players->SelectFirstItem();
    
    return true;
}

bool E_PlayerDialog::playerAdded(PG_Button* btn)
{
    SDL_Color c; c.r = 0; c.g = 0; c.b = 0;
    Player* p = Player::create("Marvin", "imperial", c, Player::AI_DUMMY);

    Playerlist::getInstance()->push_back(p);
    d_players->AddItem(p->getName().c_str(), static_cast<void*>(p));
    
    return true;
}

bool E_PlayerDialog::playerSelected(PG_ListBoxBaseItem* item)
{
    PG_Application::SetBulkMode(true);
    
    d_current = static_cast<Player*>(item->GetUserData());

    char buffer[11]; buffer[10]='\0';
    
    d_players->SetText(d_current->getName().c_str());
    switch(d_current->getType())
    {
        case Player::HUMAN:
            d_types->SetText(_("Human"));
            break;
        case Player::AI_FAST:
            d_types->SetText(_("Fast AI"));
            break;
        case Player::AI_DUMMY:
        case Player::NEUTRAL:
            d_types->SetText(_("Dummy AI"));
            break;
        case Player::AI_SMART:
            d_types->SetText(_("Smart AI"));
    }
    d_armysets->SetText(d_current->getArmyset()->getName().c_str());

    d_l_name->SetText(d_current->getName().c_str());
    snprintf(buffer, 10, "%i", d_current->getId());
    d_l_id->SetText(buffer);
    snprintf(buffer, 10, "%i", d_current->getGold());
    d_l_gold->SetText(buffer);
    snprintf(buffer, 10, "%i", d_current->getColor().r);
    d_l_red->SetText(buffer);
    snprintf(buffer, 10, "%i", d_current->getColor().g);
    d_l_green->SetText(buffer);
    snprintf(buffer, 10, "%i", d_current->getColor().b);
    d_l_blue->SetText(buffer);

    checkButtons();

    PG_Application::SetBulkMode(false);
    Update();

    return true;
}

bool E_PlayerDialog::typeChanged(PG_ListBoxBaseItem* item)
{
    if (!d_current || d_current == Playerlist::getInstance()->getNeutral())
        return true;

    std::string type(item->GetText());
    Player::TYPE kind = Player::HUMAN;
    Playerlist* pl = Playerlist::getInstance();

//    if (type == _("Human"))
//        kind = Player::HUMAN;
    if (type == _("Fast AI"))
        kind = Player::AI_FAST;
    else if (type == _("Dummy AI"))
        kind = Player::AI_DUMMY;
    else if (type == _("Smart AI"))
        kind = Player::AI_SMART;

    // create the new player (unfortunately, you cannot simply change the
    // player type)
    Player* p = Player::create(d_current->getName(), d_current->getArmyset()->getName(),
                              d_current->getColor(), kind);

    // remove the old player from the playerlist and add the new one
    for (Playerlist::iterator it = pl->begin(); it != pl->end(); it++)
        if ((*it) == d_current)
        {
            it = pl->insert(it, p);     // *it == p
            it++;               // old element is 1 behind the inserted one
            pl->fl_erase(it);
            break;
        }
    d_current = 0;
    
    int pos = 0;
    for (Playerlist::iterator it = pl->begin(); it != pl->end(); it++, pos++)
        if ((*it) == p)
            break;

    refillPlayers();
    d_players->SelectItem(pos);
    return true;
}

bool E_PlayerDialog::armysetChanged(PG_ListBoxBaseItem* item)
{
    if (!d_current)
        return true;
    
    d_current->setArmyset(std::string(item->GetText()));
    return true;
}

bool E_PlayerDialog::valueChanged(PG_LineEdit* edit)
{
    if (!d_current)
        return true;
    
    char buffer[11]; buffer[10] = '\0';
    SDL_Color c = d_current->getColor();
    
    if (edit == d_l_name)
    {
        d_current->setName(std::string(edit->GetText()));
        refillPlayers();
        d_players->SetText(edit->GetText());
    }
    else if (edit == d_l_gold)
            d_current->setGold(atoi(edit->GetText()));
    else if (edit == d_l_red || edit == d_l_green || edit == d_l_blue)
    {
            c.r = atoi(d_l_red->GetText());
            c.g = atoi(d_l_green->GetText());
            c.b = atoi(d_l_blue->GetText());
            d_current->setColor(c);

            // if the user produces out of range results, we correct him here
            snprintf(buffer, 10, "%i", d_current->getColor().r);
            d_l_red->SetText(buffer);
            snprintf(buffer, 10, "%i", d_current->getColor().g);
            d_l_green->SetText(buffer);
            snprintf(buffer, 10, "%i", d_current->getColor().b);
            d_l_blue->SetText(buffer);
    }
            
    return true;
}

void E_PlayerDialog::refillPlayers()
{
    // refill the player dropdown
    // TODO: should this become implemented in paragui, just remove the player item
    d_players->DeleteAll();

    Playerlist* pl = Playerlist::getInstance();
    for (Playerlist::iterator it = pl->begin(); it != pl->end(); it++)
        d_players->AddItem((*it)->getName().c_str(), static_cast<void*>(*it));

}

void E_PlayerDialog::checkButtons()
{
    d_b_remove->EnableReceiver(true);
    d_types->EnableReceiver(true);
    d_neutral->SetUnpressed();
    
    // neutral player may not be removed!
    if (d_current == Playerlist::getInstance()->getNeutral())
    {
        d_b_remove->EnableReceiver(false);
        d_types->EnableReceiver(false);
        d_neutral->SetPressed();
    }
}
